@@ -208,6 +208,7 @@ static const char *commands[NL80211_CMD_MAX + 1] = {
 	[NL80211_CMD_FRAME_WAIT_CANCEL] = "frame_wait_cancel",
 	[NL80211_CMD_JOIN_MESH] = "join_mesh",
 	[NL80211_CMD_LEAVE_MESH] = "leave_mesh",
+	[NL80211_CMD_SET_REKEY_OFFLOAD] = "set_rekey_offload",
 };
 
 static char cmdbuf[100];
@@ -249,8 +250,11 @@ void print_ssid_escaped(const uint8_t len, const uint8_t *data)
 	int i;
 
 	for (i = 0; i < len; i++) {
-		if (isprint(data[i]))
+		if (isprint(data[i]) && data[i] != ' ' && data[i] != '\\')
 			printf("%c", data[i]);
+		else if (data[i] == ' ' &&
+			 (i != 0 && i != len -1))
+			printf(" ");
 		else
 			printf("\\x%.2x", data[i]);
 	}
